<?php
/**
 * NOTICE OF LICENSE.
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author    Propoza <suppport@propoza.com>
 * @copyright Copyright © 2016 Propoza
 * @license   LICENSE.txt
 */

/**
 * Class PropozaAddProposalModuleFrontController.
 */
class PropozaAddProposalModuleFrontController extends ModuleFrontController
{
    /**
     * @throws Exception
     */
    public function postProcess()
    {
        if (!(boolean) Configuration::get('PROPOZA_ENABLED')) {
            throw new Exception('Propoza is disabled');
        }
        $post = Tools::jsonDecode(Tools::file_get_contents('php://input'), true);
        $propozaQuoteId = $post['id'];
        $cartId = $post['shop_quote_id'];
        $proposalTotal = $post['total_proposal_price'];
        header('Content-Type: application/json');
        if (isset($cartId) &&
            isset($proposalTotal) &&
            isset($propozaQuoteId)
        ) {
            $cart = new Cart($cartId);
            $idCartRule = CartRule::getCartRuleIdByPropozaQuoteId($propozaQuoteId);

            if ($idCartRule) {
                $cartRule = CartRule::updateProposalCartRule($cartId, $idCartRule, $proposalTotal);
            } else {
                $cartRule = CartRule::createProposalCartRule($cartId, $propozaQuoteId, $proposalTotal);
            }
            if (!Validate::isLoadedObject($cartRule)) {
                throw new Exception('Cart rule could not be found');
            }

            if (!Validate::isLoadedObject($cart)) {
                throw new Exception('Cart could not be found');
            }
            CartRule::removeCartClonesForPropozaQuote($propozaQuoteId);
            $clonedCart = $cart->duplicate();
            if (!$clonedCart['success']) {
                throw new Exception('Cart could not be duplicated');
            }

            $clonedCart['cart']->addCartRule($cartRule->id);

            echo Tools::jsonEncode(array('quote_id' => $clonedCart['cart']->id));
        } else {
            throw new Exception('Incorrect parameters');
        }
        die;
    }
}
