<?php
/**
 * NOTICE OF LICENSE.
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author    Propoza <suppport@propoza.com>
 * @copyright Copyright © 2016 Propoza
 * @license   LICENSE.txt
 */

/**
 * Class CartRule.
 */
class CartRule extends CartRuleCore
{
    public $includingTax = true;

    /**
     * @param $cartId
     * @param $propozaQuoteId
     * @param $proposalTotal
     *
     * @return bool|CartRule
     */
    public static function createProposalCartRule($cartId, $propozaQuoteId, $proposalTotal)
    {
        $cart = new Cart($cartId);
        $cartRule = new self();
        $subTotal = $cart->getOrderTotal($cartRule->includingTax, Cart::ONLY_PRODUCTS);
        $discount = $subTotal -
            $proposalTotal;
        $cartRule->code = $cartRule->generateCouponCode(8);
        $cartRule->name = array(
            Configuration::get('PS_LANG_DEFAULT') => sprintf('Proposal Discount for Quote #%s', $propozaQuoteId),
        );
        $now = time();
        $cartRule->date_from = date('Y-m-d H:i:s', $now);
        $cartRule->date_to = date(
            'Y-m-d H:i:s',
            $now + (3600 * 24 * 365.25)
        ); /* 1 year */
        $cartRule->partial_use = 0;
        $cartRule->quantity = 1;
        $cartRule->quantity_per_user = 1;
        $cartRule->reduction_currency = $cart->id_currency;
        $cartRule->reduction_tax = $cartRule->includingTax;
        $cartRule->reduction_amount = $discount;
        $cartRule->minimum_amount_tax = $cartRule->includingTax;
        $cartRule->minimum_amount = $subTotal;
        $cartRule->minimum_amount_currency = $cart->id_currency;
        $cartRule->product_restriction = 1;
        $cartRule->active = 1;

        if ($cartRule->save() &&
            Db::getInstance()->insert(
                'propoza_quote_cart_rule',
                array('id_propoza_quote' => $propozaQuoteId, 'id_cart_rule' => $cartRule->id)
            )
        ) {
            $products = $cart->getProducts();
            Db::getInstance()->execute(
                'INSERT INTO `'._DB_PREFIX_.'cart_rule_group` (`id_cart_rule`, `id_group`) 
                VALUES ('.(int) $cartRule->id.','. 3 .')'
            );
            foreach ($products as $product) {
                Db::getInstance()->execute(
                    'INSERT INTO `'._DB_PREFIX_.'cart_rule_product_rule_group` (`id_cart_rule`, `quantity`) 
                    VALUES ('.(int) $cartRule->id.', '.(int) $product['quantity'].')'
                );
                $id_product_rule_group = Db::getInstance()->Insert_ID();
                Db::getInstance()->execute(
                    'INSERT INTO `'._DB_PREFIX_.'cart_rule_product_rule` (`id_product_rule_group`, `type`) 
                    VALUES ('.(int) $id_product_rule_group.', "products")'
                );
                $id_product_rule = Db::getInstance()->Insert_ID();

                Db::getInstance()->execute(
                    'INSERT INTO `'._DB_PREFIX_.'cart_rule_product_rule_value` (`id_product_rule`, `id_item`) 
                    VALUES ('.(int) $id_product_rule.','.(int) $product['id_product'].')'
                );
            }

            return $cartRule;
        }

        return false;
    }

    /**
     * @param $length
     *
     * @return string
     */
    public function generateCouponCode($length)
    {
        $chars = '123456789ABCDEFGHIJKLMNPQRSTUVWXYZ';
        $code = '';
        for ($i = 0; $i <
        $length; ++$i) {
            $code .= $chars[rand(0, Tools::strlen($chars) - 1)];
        }

        return $code;
    }

    /**
     * @param $cartId
     * @param $idCartRule
     * @param $proposalTotal
     *
     * @return bool|CartRule
     */
    public static function updateProposalCartRule($cartId, $idCartRule, $proposalTotal)
    {
        $cart = new Cart($cartId);
        $cartRule = new self($idCartRule);
        $subTotal = $cart->getOrderTotal($cartRule->includingTax, Cart::ONLY_PRODUCTS);
        $discount = $subTotal -
            $proposalTotal;
        $cartRule->reduction_amount = $discount;
        $cartRule->minimum_amount = $subTotal;
        if ($cartRule->update()) {
            return $cartRule;
        }

        return false;
    }

    /**
     * @param $propozaQuoteId
     *
     * @return int
     */
    public static function getCartRuleIdByPropozaQuoteId($propozaQuoteId)
    {
        $idCartRule = (int) Db::getInstance()->getValue(
            'SELECT pqrc.id_cart_rule FROM '._DB_PREFIX_.'propoza_quote_cart_rule pqrc 
            WHERE pqrc.id_propoza_quote = '.(int) $propozaQuoteId
        );

        return $idCartRule;
    }

    /**
     * @param $propozaQuoteId
     *
     * @throws PrestaShopDatabaseException
     *
     * @return bool
     */
    public static function removeCartClonesForPropozaQuote($propozaQuoteId)
    {
        $result = Db::getInstance()->executeS(
            'SELECT cd.`id_cart` FROM `'._DB_PREFIX_.'cart_cart_rule` cd 
            LEFT JOIN `'._DB_PREFIX_.'propoza_quote_cart_rule` cr ON cd.`id_cart_rule` = cr.`id_cart_rule` 
            WHERE `id_propoza_quote` = '.(int) $propozaQuoteId
        );
        $deleted = true;
        foreach ($result as $row) {
            $cart = new Cart($row['id_cart']);
            $deleted = $cart->delete();
        }

        return $deleted;
    }

    /**
     * @param Cart $cart
     *
     * @return bool
     */
    public static function cartHasPropozaCoupon(Cart $cart)
    {
        foreach ($cart->getCartRules() as $cartRule) {
            if ($cartRule['obj']->isPropozaCoupon()) {
                return true;
            }
        }

        return false;
    }

    /**
     * @return bool
     */
    public function delete()
    {
        return parent::delete() &&
        Db::getInstance()->delete('propoza_quote_cart_rule', '`id_cart_rule` = '.(int) $this->id);
    }

    /**
     * @throws PrestaShopDatabaseException
     */
    public function getPropozaQuoteId()
    {
        $dbquery = new DbQuery();
        $dbquery->select('id_propoza_quote');
        $dbquery->from('propoza_quote_cart_rule', 'pqcr');
        $dbquery->where(sprintf('pqcr.`id_cart_rule` = %s', (int) $this->id));
        $cartRules = Db::getInstance()->executeS($dbquery->build());

        if (count($cartRules) >=
            1
        ) {
            return $cartRules[0]['id_propoza_quote'];
        }

        return null;
    }

    /**
     * @param Context    $context
     * @param bool|false $alreadyInCart
     * @param bool|true  $display_error
     * @param bool|true  $check_carrier
     *
     * @return array|bool|string
     */
    public function checkValidity(
        Context $context,
        $alreadyInCart = false,
        $display_error = true,
        $check_carrier = true
    ) {
        //Added for Propoza
        if ($this->isPropozaCoupon()) {
            if ($this->minimum_amount &&
                $check_carrier
            ) {
                // Minimum amount is converted to the contextual currency
                $minimum_amount = $this->minimum_amount;
                if ($this->minimum_amount_currency !=
                    Context::getContext()->currency->id
                ) {
                    $minimum_amount = Tools::convertPriceFull(
                        $minimum_amount,
                        new Currency($this->minimum_amount_currency),
                        Context::getContext()->currency
                    );
                }

                $cartTotal = $context->cart->getOrderTotal($this->minimum_amount_tax, Cart::ONLY_PRODUCTS);

                if ($cartTotal !=
                    $minimum_amount
                ) {
                    return (!$display_error) ? false :
                        Tools::displayError('You have not reached the amount required to use this voucher');
                }
            }
        }//END

        return parent::checkValidity($context, $alreadyInCart, $display_error, $check_carrier);
    }

    /**
     * @return bool
     *
     * @throws PrestaShopDatabaseException
     */
    public function isPropozaCoupon()
    {
        $dbquery = new DbQuery();
        $dbquery->select('id_propoza_quote');
        $dbquery->from('propoza_quote_cart_rule', 'pqcr');
        $dbquery->where(sprintf('pqcr.`id_cart_rule` = %s', (int) $this->id));
        $cartRules = Db::getInstance()->executeS($dbquery->build());

        return count($cartRules) >
        0;
    }

    /**
     * @param Context    $context
     * @param bool|false $return_products
     * @param bool|true  $display_error
     * @param bool|false $already_in_cart
     *
     * @return array|bool|string
     *
     * @throws PrestaShopDatabaseException
     */
    protected function checkProductRestrictions(
        Context $context,
        $return_products = false,
        $display_error = true,
        $already_in_cart = false
    ) {
        //Added for Propoza
        if ($this->isPropozaCoupon()) {
            // Check if the products chosen by the customer are usable with the cart rule
            if ($this->product_restriction) {
                $product_rule_groups = $this->getProductRuleGroups();
                foreach ($product_rule_groups as $id_product_rule_group => $product_rule_group) {
                    $eligible_products_list = array();
                    if (isset($context->cart) &&
                        is_object($context->cart) &&
                        is_array($products = $context->cart->getProducts())
                    ) {
                        foreach ($products as $product) {
                            $eligible_products_list[] = (int) $product['id_product'].
                                '-'.
                                (int) $product['id_product_attribute'];
                        }
                    }
                    if (!count($eligible_products_list)) {
                        return (!$display_error) ? false :
                            Tools::displayError('You cannot use this voucher in an empty cart');
                    }

                    $product_rules = $this->getProductRules($id_product_rule_group);
                    foreach ($product_rules as $product_rule) {
                        switch ($product_rule['type']) {
                            case 'products':
                                $cart_products = Db::getInstance()->executeS(
                                    'SELECT cp.quantity, cp.`id_product`FROM `'._DB_PREFIX_.'cart_product` cp
                                    WHERE cp.`id_cart` = '.(int) $context->cart->id.' 
                                    AND cp.`id_product` 
                                    IN ('.implode(',', array_map('intval', $eligible_products_list)).')'
                                );
                                $count_matching_products = 0;
                                $matching_products_list = array();
                                foreach ($cart_products as $cart_product) {
                                    if (in_array($cart_product['id_product'], $product_rule['values'])) {
                                        $count_matching_products += $cart_product['quantity'];
                                        if ($already_in_cart &&
                                            $this->gift_product ==
                                            $cart_product['id_product']
                                        ) {
                                            --$count_matching_products;
                                        }
                                        $matching_products_list[] = $cart_product['id_product'].
                                            '-0';
                                    }
                                }

                                if ($count_matching_products !=
                                    $product_rule_group['quantity']
                                ) {
                                    return (!$display_error) ? false :
                                        Tools::displayError('You cannot use this voucher with these products');
                                }

                                break;
                        }
                    }
                }
            }
        }

        //END
        return parent::checkProductRestrictions($context, $return_products, $display_error, $already_in_cart);
    }
}
