<?php
/**
 * NOTICE OF LICENSE.
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author    Propoza <suppport@propoza.com>
 * @copyright Copyright © 2016 Propoza
 * @license   LICENSE.txt
 */

/**
 * Class PropozaHelper.
 */
class PropozaHelper
{
    /**
     * @param $string
     *
     * @return int
     */
    public static function isValidApiKey($string)
    {
        return preg_match('/^[A-Za-z0-9+\/]{226}==$/', $string);
    }

    /**
     * @return string
     */
    public static function getSignUpPropozaUrl()
    {
        return sprintf('%s%s/accounts/create?client=prestashop', self::getProtocol(), self::getPropozaUrl());
    }

    /**
     * @return string
     */
    public static function getProtocol()
    {
        return 'https://';
    }

    /**
     * @return string
     */
    public static function getPropozaUrl()
    {
        return 'propoza.com';
    }

    /**
     * @return string
     */
    public static function getQuoteRequestFormUrl()
    {
        return sprintf('%s/api/PrestashopQuotes/quoteRequestForm', self::getDashboardPropozaUrl());
    }

    /**
     * @param null $subDomain
     *
     * @return string
     */
    public static function getDashboardPropozaUrl($subDomain = null)
    {
        if ($subDomain ===
            null
        ) {
            $subDomain = Configuration::get('PROPOZA_DOMAIN');
        }

        return sprintf('%s%s.%s', self::getProtocol(), $subDomain, self::getPropozaUrl());
    }

    /**
     * @return string
     */
    public static function getQuoteOrderedUrl()
    {
        return sprintf('%s/api/PrestashopQuotes/edit.json', self::getDashboardPropozaUrl());
    }

    /**
     * @return string
     */
    public static function getRequestFormCssUrl()
    {
        return sprintf('%s/css/Quotes/request-form.css', self::getDashboardPropozaUrl());
    }

    /**
     * @return string
     */
    public static function getRequestFormJsUrl()
    {
        return sprintf('%s/js/Quotes/request-form.js', self::getDashboardPropozaUrl());
    }

    /**
     * @return bool
     */
    public static function configured()
    {
        $domain = Configuration::get('PROPOZA_DOMAIN');
        $apiKey = Configuration::get('API_KEY');

        return isset($apiKey) &&
            isset($domain) &&
            trim($apiKey) !==
            '' &&
            trim($domain) !==
            '';
    }

    /**
     * @param null $subDomain
     * @param null $token
     *
     * @return string
     */
    public static function getDashboardPropozaLoginUrl($subDomain = null, $token = null)
    {
        if (isset($token)) {
            $token = sprintf('?_token=%s', $token);
        }

        return sprintf('%s/login%s', self::getDashboardPropozaUrl($subDomain), $token);
    }

    /**
     * @param null $subDomain
     *
     * @return string
     */
    public static function getDashboardPropozaTokenUrl($subDomain = null)
    {
        return sprintf('%s/api/MerchantsApi/token.json', self::getDashboardPropozaUrl($subDomain));
    }

    /**
     * @param null $apiKey
     * @param null $subDomain
     *
     * @return mixed
     */
    public function testConnection($apiKey = null, $subDomain = null)
    {
        if ($subDomain ===
            null
        ) {
            $subDomain = Configuration::get('PROPOZA_DOMAIN');
        }

        if ($apiKey ===
            null
        ) {
            $apiKey = Configuration::get('API_KEY');
        }

        $process = curl_init(sprintf($this->getConnectionTestUrl(), $subDomain));
        curl_setopt($process, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
                'Authorization: Basic '.
                $apiKey,
            ));
        curl_setopt($process, CURLOPT_HEADER, false);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, true);
        if (defined('_PS_MODE_DEV_') &&
            _PS_MODE_DEV_
        ) {
            curl_setopt($process, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($process, CURLOPT_SSL_VERIFYPEER, false);
        }
        $response = curl_exec($process);
        if ($error = curl_error($process)) {
            throw new Exception(curl_error($error));
        }
        curl_close($process);

        return $response;
    }

    /**
     * @return string
     */
    public static function getConnectionTestUrl()
    {
        return sprintf('%s/api/PrestashopQuotes/testConnection.json', self::getDashboardPropozaUrl('%s'));
    }
}
