<?php
/**
 * NOTICE OF LICENSE.
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author    Propoza <suppport@propoza.com>
 * @copyright Copyright © 2016 Propoza
 * @license   LICENSE.txt
 */

/**
 * Class PropozaQuoteRequest.
 */
class PropozaQuoteRequest
{
    protected $include_default_store_tax = true;

    /**
     * @return mixed
     */
    public function displayContent()
    {
        if (PropozaHelper::configured()) {
            $path = Module::getInstanceByName('propoza')->getLocalPath().
                'views/templates/hook/propoza.tpl';

            return Context::getContext()->smarty->fetch($path);
        }
    }

    /**
     * @return mixed
     *
     * @throws Exception
     */
    public function getRequestQuoteFormHtml()
    {
        $clonedCart = Context::getContext()->cart->duplicate();
        if ($clonedCart['success']) {
            $process = curl_init(PropozaHelper::getQuoteRequestFormUrl());
            curl_setopt($process, CURLOPT_HTTPHEADER, array(
                    'Content-Type: application/json',
                    'Authorization: Basic '.
                    Configuration::get('API_KEY'),
                ));
            curl_setopt($process, CURLOPT_HEADER, false);
            curl_setopt($process, CURLOPT_POST, true);
            curl_setopt($process, CURLOPT_POSTFIELDS, Tools::jsonEncode($this->getPreparedQuote($clonedCart['cart'])));
            curl_setopt($process, CURLOPT_FOLLOWLOCATION, true);
            if (defined('_PS_MODE_DEV_') &&
                _PS_MODE_DEV_
            ) {
                curl_setopt($process, CURLOPT_SSL_VERIFYHOST, false);
                curl_setopt($process, CURLOPT_SSL_VERIFYPEER, false);
            }
            curl_setopt($process, CURLOPT_RETURNTRANSFER, true);
            $response = curl_exec($process);
            curl_close($process);

            return $response;
        }
        throw new Exception('Could not duplicate cart');
    }

    /**
     * @param Cart $cart
     *
     * @return array
     */
    public function getPreparedQuote(Cart $cart)
    {
        return $this->prepareQuoteRequest($cart);
    }

    /**
     * @param Cart $cart
     *
     * @return array
     */
    public function prepareQuoteRequest(Cart $cart)
    {
        $toPropozaQuote = array();
        $toPropozaQuote['Quote'] = $this->prepareQuote($cart);
        $toPropozaQuote['Quote']['shop_url'] = Context::getContext()->link->getPageLink('');
        $toPropozaQuote['Quote']['Requester'] = $this->prepareRequester($cart);
        $toPropozaQuote['Quote']['Product'] = $this->prepareProducts($cart);

        return $toPropozaQuote;
    }

    /**
     * @param Cart $cart
     *
     * @return array
     */
    private function prepareQuote(Cart $cart)
    {
        $toPropozaQuote = array();
        $toPropozaQuote['shop_quote_id'] = $cart->id;
        $toPropozaQuote['cart_currency'] = Currency::getCurrencyInstance((int) $cart->id_currency)->iso_code;
        $toPropozaQuote['include_default_store_tax'] = $this->include_default_store_tax;

        return $toPropozaQuote;
    }

    /**
     * @param Cart $cart
     *
     * @return array
     */
    public function prepareRequester(Cart $cart)
    {
        $requester = array();
        $requester['firstname'] = $this->getCurrentLoggedInCustomer($cart)->firstname;
        $requester['lastname'] = $this->getCurrentLoggedInCustomer($cart)->lastname;
        $requester['email'] = $this->getCurrentLoggedInCustomer($cart)->email;
        $requester['company'] = $this->getCurrentLoggedInCustomer($cart)->company;

        return $requester;
    }

    /**
     * @param Cart $cart
     *
     * @return Customer
     */
    private function getCurrentLoggedInCustomer(Cart $cart)
    {
        return new Customer((int) $cart->id_customer);
    }

    /**
     * @param Cart $cart
     *
     * @return array
     */
    private function prepareProducts(Cart $cart)
    {
        $products = array();
        $count = 0;
        foreach ($cart->getProducts(true) as $product) {
            $products[$count] = $this->prepareProduct($product);
            ++$count;
        }

        return $products;
    }

    /**
     * @param $product
     *
     * @return array
     */
    private function prepareProduct($product)
    {
        $propozaProduct = array();
        $propozaProduct['name'] = $product['name'];
        //price_wt is used for prices including tax else use price and set value include_default_store_tax to false
        $propozaProduct['original_price'] = $this->include_default_store_tax ? $product['price_wt'] : $product['price'];
        $propozaProduct['sku'] = $product['reference'];
        $propozaProduct['quantity'] = $product['quantity'];
        $propozaProduct['ProductAttribute'] = $this->prepareProductAttributes($product);

        return $propozaProduct;
    }

    /**
     * @param $product
     *
     * @return array
     */
    private function prepareProductAttributes($product)
    {
        $productAttributes = array();
        $productObject = new Product($product['id_product'], Context::getContext()->language->id);
        $productAttributeId = $product['id_product_attribute'];
        $combinations = $productObject->getAttributeCombinationsById(
            $productAttributeId,
            Context::getContext()->language->id
        );
        $productAttributeCombinations = $combinations;
        $counter = 0;
        foreach ($productAttributeCombinations as $attributeCombination) {
            $productAttributes[$counter]['name'] = $attributeCombination['group_name'];
            $productAttributes[$counter]['value'] = $attributeCombination['attribute_name'];
            $productAttributes[$counter]['price'] = $attributeCombination['price'];
            ++$counter;
        }

        return $productAttributes;
    }

    /**
     * @return array
     */
    public function getPreparedLoggedInUser()
    {
        return $this->prepareRequester();
    }

    /**
     * @param $formAction
     * @param $formData
     *
     * @return mixed
     */
    public function requestQuoteFormSubmit($formAction, $formData)
    {
        $process = curl_init($formAction);
        curl_setopt($process, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
                'Authorization: Basic '.
                Configuration::get('API_KEY'),
            ));
        curl_setopt($process, CURLOPT_HEADER, false);
        curl_setopt($process, CURLOPT_POST, true);
        curl_setopt($process, CURLOPT_POSTFIELDS, Tools::jsonEncode($formData));
        curl_setopt($process, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, true);
        if (defined('_PS_MODE_DEV_') &&
            _PS_MODE_DEV_
        ) {
            curl_setopt($process, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($process, CURLOPT_SSL_VERIFYPEER, false);
        }
        $response = curl_exec($process);
        curl_close($process);

        return $response;
    }

    /**
     * @param $quoteId
     */
    public function updateStatusOrdered($quoteId)
    {
        $process = curl_init(PropozaHelper::getQuoteOrderedUrl());
        curl_setopt($process, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
                'Authorization: Basic '.
                Configuration::get('API_KEY'),
            ));
        curl_setopt($process, CURLOPT_HEADER, false);
        curl_setopt($process, CURLOPT_POST, true);
        curl_setopt(
            $process,
            CURLOPT_POSTFIELDS,
            Tools::jsonEncode(
                array(
                    'Quote' => array(
                        'id' => $quoteId,
                        'main_status_id' => 2,
                        'sub_status_id' => 5,
                    ),
                )
            )
        );
        curl_setopt($process, CURLOPT_FOLLOWLOCATION, true);
        if (defined('_PS_MODE_DEV_') && _PS_MODE_DEV_) {
            curl_setopt($process, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($process, CURLOPT_SSL_VERIFYPEER, false);
        }
        curl_setopt($process, CURLOPT_RETURNTRANSFER, true);
        curl_exec($process);

        $responseType = floor(curl_getinfo($process, CURLINFO_HTTP_CODE) / 100);
        if ($responseType !== 1 || $responseType !== 2) {
            //TODO Log error
        }
        $cartRuleId = CartRule::getCartRuleIdByPropozaQuoteId($quoteId);
        $cartRule = new CartRule($cartRuleId);
        $cartRule->delete();
        curl_close($process);
    }
}
