<?php
/**
 * NOTICE OF LICENSE.
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author    Propoza <suppport@propoza.com>
 * @copyright Copyright © 2016 Propoza
 * @license   LICENSE.txt
 */

/**
 * Class Propoza.
 */
class Propoza extends Module
{
    /**
     * @var bool
     */
    protected $config_form = false;

    /**
     *
     */
    public function __construct()
    {
        include_once dirname(__FILE__).
            '/propoza-helper.php';
        include_once dirname(__FILE__).
            '/propoza-quote-request.php';

        $this->name = 'propoza';
        $this->tab = 'billing_invoicing';
        $this->version = '1.0.8';
        $this->author = 'Propoza';
        $this->need_instance = 0;
        $this->module_key = 'f64b40eddebcd9779887fe7bc0845da0';

        /*
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Propoza');
        $this->description = $this->l(
            'Propoza adds quotations functionality to your webshop. This means more leads & more order!'
        );

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall Propoza?');
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update.
     */
    public function install()
    {
        include dirname(__FILE__).
            '/sql/install.php';
        $tab = new Tab();
        $tab->active = 0;
        $tab->class_name = 'AdminPropoza';
        $tab->name = array();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = 'Quotes';
        }

        $tab->module = $this->name;

        if (!$tab->add() ||
            !parent::install() ||
            !$this->registerHook('header') ||
            !$this->registerHook('backOfficeHeader') ||
            !$this->registerHook('displayShoppingCart') ||
            !$this->registerHook('displayBeforeShoppingCartBlock') ||
            !$this->registerHook('newOrder')
        ) {
            return false;
        }

        return true;
    }

    /**
     * @return bool
     */
    public function uninstall()
    {
        include dirname(__FILE__).
            '/sql/uninstall.php';

        $id_tab = (int) Tab::getIdFromClassName('AdminPropoza');

        if ($id_tab) {
            $tab = new Tab($id_tab);
            $tab->delete();
        }

        if (!parent::uninstall() ||
            !Configuration::deleteByName('API_KEY') ||
            !Configuration::deleteByName('PROPOZA_ENABLED') ||
            !Configuration::deleteByName('PROPOZA_DOMAIN')
        ) {
            return false;
        }

        return true;
    }

    /**
     * Load the configuration form.
     */
    public function getContent()
    {
        $this->context->smarty->assign('module_dir', $this->_path);

        $message = '';
        if (Tools::isSubmit('submit'.$this->name)) {
            $message = $this->saveContent();
        }

        $this->displayContent($message);
        $output = $this->context->smarty->fetch(
            $this->local_path.'views/templates/admin/configure.tpl'
        );

        return $output;
    }

    /**
     * Save configuration Propoza.
     *
     * @return mixed
     */
    private function saveContent()
    {
        $message = '';

        $apiKey = (String) Tools::getValue('API_KEY');

        if (!$apiKey ||
            empty($apiKey) ||
            !PropozaHelper::isValidApiKey($apiKey)
        ) {
            $message .= $this->displayError($this->l('Invalid api key value'));
        } else {
            Configuration::updateValue('API_KEY', $apiKey);
        }

        $subDomain = (String) Tools::getValue('PROPOZA_DOMAIN');
        if (!$subDomain ||
            empty($subDomain) ||
            !Validate::isGenericName($subDomain)
        ) {
            $message .= $this->displayError($this->l('Invalid propoza domain'));
        } else {
            Configuration::updateValue('PROPOZA_DOMAIN', $subDomain);
        }

        Configuration::updateValue('PROPOZA_ENABLED', (boolean) Tools::getValue('PROPOZA_ENABLED'));

        if ((boolean) Configuration::get('PROPOZA_ENABLED') &&
            PropozaHelper::configured()
        ) {
            Tab::enablingForModule($this->name);
        } else {
            Tab::disablingForModule($this->name);
        }

        if (empty($message)) {
            $message .= $this->displayConfirmation($this->l('Settings updated'));
        }

        return $message;
    }

    /**
     * Add display content to configuration Propoza.
     *
     * @param $message
     */
    private function displayContent($message)
    {
        $this->context->smarty->assign(array('MESSAGE' => $message));
        $this->context->smarty->assign(array('TOKEN' => Tools::getAdminTokenLite('AdminModules')));
        $this->context->smarty->assign(array('API_KEY' => Configuration::get('API_KEY')));
        $this->context->smarty->assign(array('PROPOZA_DOMAIN' => Configuration::get('PROPOZA_DOMAIN')));
        $this->context->smarty->assign(array('PROPOZA_ENABLED' => (bool) Configuration::get('PROPOZA_ENABLED')));
        $this->context->smarty->assign(array('DASHBOARD_PROPOZA_URL' => PropozaHelper::getDashboardPropozaUrl('%s')));
        $this->context->smarty->assign(array('SIGN_UP_URL' => PropozaHelper::getSignUpPropozaUrl()));
        $apiKey = Configuration::get('API_KEY');
        $this->context->smarty->assign(array('IS_VALID_API_KEY' => PropozaHelper::isValidApiKey($apiKey)));
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        $this->context->controller->addJS($this->_path.'/views/js/back.js');
        $this->context->controller->addCSS($this->_path.'/views/css/back.css');
        $this->context->controller->addJS('//ajax.cdnjs.com/ajax/libs/json2/20110223/json2.min.js');
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path.'/views/js/front.js');
        $this->context->controller->addCSS(PropozaHelper::getRequestFormCssUrl());
        $this->context->controller->addJS(PropozaHelper::getRequestFormJsUrl());
        $this->context->controller->addCSS($this->_path.'/views/css/front.css');
        $this->context->controller->addJS($this->_path.'/views/js/SerializeObject.js');
        $this->context->controller->addJS('//ajax.cdnjs.com/ajax/libs/json2/20110223/json2.min.js');
    }

    /**
     */
    public function hookDisplayShoppingCart()
    {
        if (Configuration::get('PROPOZA_ENABLED')) {
            if (!CartRule::cartHasPropozaCoupon(Context::getContext()->cart)) {
                $propozaQuoteRequest = new PropozaQuoteRequest();

                return $propozaQuoteRequest->displayContent();
            }
        }
    }

    /**
     */
    public function hookDisplayBeforeShoppingCartBlock()
    {
        if (Configuration::get('PROPOZA_ENABLED')) {
            if (CartRule::cartHasPropozaCoupon(Context::getContext()->cart)) {
                return $this->displayConfirmation(
                    '<b>'.$this->l('Great: ').'</b><i>'.
                    $this->l('Your proposal discount has successfully been applied. Check the new Total.').
                    '</i>'
                ).$this->displayWarning(
                    '<b>'.$this->l('Please note:').'</b><i>'.
                    $this->l('Changing the cart contents will invalidate the proposal price.').'</i>'
                );
            }
        }
    }

    /**
     * @param $params
     */
    public function hookNewOrder($params)
    {
        if (CartRule::cartHasPropozaCoupon($params['cart'])) {
            $propozaQuoteRequest = new PropozaQuoteRequest();
            foreach ($params['cart']->getCartRules() as $cartRule) {
                if ($cartRule['obj']->isPropozaCoupon()) {
                    $propozaQuoteRequest->updateStatusOrdered($cartRule['obj']->getPropozaQuoteId());
                }
            }
        }
    }
}
