/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author    Propoza <suppport@propoza.com>
 * @copyright Copyright © 2016 Propoza
 * @license   LICENSE.txt
 */

(function ($) {
    $(function () {
        var api_key_element = $('#API_KEY');
        var sub_domain_element = $('#PROPOZA_DOMAIN');
        var propoza_dashboard_link = $('.dashboard_propoza_link');
        var test_connection_button = $('#testconnection_button');
        var loader_container = $('#loader_container');

        propoza_dashboard_link.hideOnEmptyValue(sub_domain_element.attr('id'));
        test_connection_button.hideOnEmptyValue([api_key_element.attr('id'), sub_domain_element.attr('id')]);

        sub_domain_element.on('input', function () {
            propoza_dashboard_link.hideOnEmptyValue(sub_domain_element.attr('id'));
            test_connection_button.hideOnEmptyValue([api_key_element.attr('id'), sub_domain_element.attr('id')]);

            /**
             * get sub-domain from input value if entered a full url e.g.
             * demo.propoza.com
             * demo.local.propoza.com
             * http://demo.propoza.com
             * http://demo.local.propoza.com
             * https://demo.propoza.com
             * https://demo.local.propoza.com
             *
             * @type {RegExp}
             */
            var regex = /(http[s]?:\/\/)?(.*)\.propoza\.com/g;
            var result = regex.exec(sub_domain_element.val());
            if (result !== null) {
                sub_domain_element.val(result[2]);
            }
        });

        api_key_element.on('input', function () {
            test_connection_button.hideOnEmptyValue([api_key_element.attr('id'), sub_domain_element.attr('id')]);
        });

        propoza_dashboard_link.click(function (e) {
            window.open(propoza_dashboard_link.attr('href').replace('%s', sub_domain_element.val()));
            e.preventDefault();
        });

        test_connection_button.click(function () {
            loader_container.show();
            $.post('../modules/propoza/propoza-ajax.php', {
                'action': 'testConnection',
                'api_key': api_key_element.val(),
                'sub_domain': sub_domain_element.val()
            }, function (data) {
                loader_container.hide();
                showMessage(data.response);
            }, 'json').fail(function () {
                loader_container.hide();
                showMessage(false);
            });
        });

        function showMessage(success) {
            if (success === true) {
                alert('Test connection success!');
            } else {
                alert('Test connection failed!');
            }
        }
    });

    $.fn.hideOnEmptyValue = function (elementIds) {
        if ($.isArray(elementIds)) {
            hasValue = true;
            $.each(elementIds, function (index, value) {
                if ($('#' + value).val() == '') {
                    return hasValue = false;
                }
            });
        }
        else {
            hasValue = $('#' + elementIds).val() !== '';
        }
        if (hasValue) {
            $(this).show();
        } else {
            $(this).hide();
        }
    }
})(jQuery);